import java.util.*;
import java.util.StringTokenizer;
import java.io.*;

public class SpellCheck
{
    Hashtable words;
    String highlightLeft = "<b>";
    String highlightRight = "</b>";
    public SpellCheck()
    {
        words = new Hashtable(50000);
    }
    public String loadDictionary(String filename)
    {
        String ret = "";
        try
        {
            java.io.File f = new File(filename);
            DataInputStream dis = new DataInputStream(new FileInputStream(f));
            boolean reachedEnd = false;
            byte[] b = new byte[(int)f.length()];
            int incCount = 0;
            int count = 0;
            while (!reachedEnd && ((count = dis.read(b, 0, b.length)) > 0))
            {
                while (count < b.length && !reachedEnd)
                {
                    incCount = dis.read(b, count, b.length - count);
                    if (incCount > 0)
                    {
                        count += incCount;
                    } else
                        reachedEnd = true;
                }
            }
            String s = new String(b);
            StringTokenizer st = new StringTokenizer(s, "\r\n");
            while (st.hasMoreTokens())
            {
                words.put(st.nextToken(), "");
            }
        }
        catch (Exception e) {
                ret = e.toString();
            }
        return ret;
    }
    public String breakWords(String s)
    {
		String retVal = "";
		boolean inTag = false;
		s = s.toLowerCase();
		char ch;
		for (int w = 0; w < s.length(); w++)
		{
			ch = s.charAt(w);
			
			if (ch == '<') 
			{
				inTag = true;
				retVal = retVal + " ";
			} else if (ch == '>') {
			    inTag = false;
				retVal = retVal + " ";
			} else if ((!inTag) && (ch >= 'a' && ch <= 'z')) { 
				retVal = retVal + ch;
			} else
				retVal = retVal + " ";
		}
		return retVal;
    }
    public String getBadWords(String s)
    {
        String badWords = "";
        Hashtable hash = new Hashtable();
        StringTokenizer st = new StringTokenizer(breakWords(s), " ");
        while (st.hasMoreTokens())
        {
            String w = st.nextToken();
            if (w.length() > 1 && !words.containsKey(w))
                hash.put(w, "");
        }
        for (Enumeration e = hash.keys() ; e.hasMoreElements() ;) {
            badWords += e.nextElement() + ",";
         }
         return badWords;
    }
    public String highlightWords(String phrase, String misspellings)
    {
		if (misspellings.length() == 0) 
		    return phrase;

		StringTokenizer st = new StringTokenizer(misspellings, ",");
        Hashtable dic = new Hashtable();
        while (st.hasMoreTokens())
            dic.put(st.nextToken(), "");

        String s2 = "";
		boolean inTag = false;
		phrase = " " + phrase + " ";
		String posPhrase = breakWords(phrase);

        
        for (Enumeration e = dic.keys() ; e.hasMoreElements() ;) 
        {
            String word = (String)e.nextElement();
			int pos = posPhrase.indexOf(" " + word + " ");
			
			while (pos >= 0 )
			{
				String leftPhrase = phrase.substring(0, pos+1);
				String rightPhrase = phrase.substring(pos + word.length() +  1);
				
				String leftPosPhrase = posPhrase.substring(0, pos+1);
				String rightPosPhrase = posPhrase.substring(pos + word.length() +  1);

				String hit = phrase.substring(pos+1, pos+1+word.length());
				phrase = leftPhrase + highlightLeft + hit + highlightRight + rightPhrase;
				posPhrase = leftPosPhrase + highlightLeft + hit + highlightRight + rightPosPhrase;
				int start = leftPhrase.length() + hit.length() + highlightLeft.length() + highlightRight.length();
				pos = posPhrase.indexOf(" " + word + " ", start);
				//System.out.println(posPhrase);
				//System.out.println(phrase);
			}
        }
		return phrase;
    }
    public void setHighlights(String s1, String s2)
    {
        this.highlightLeft = s1;
        this.highlightRight = s2;
    }
    public int getWordCount()
    {
        return words.size();
    }
    public String checkSpelling(String p)
    {
        String ms = getBadWords(p);
        return highlightWords(p, ms);
    }
    public static void main(String args[])
    {
        SpellCheck sc = new SpellCheck();
        sc.loadDictionary("c:\\temp\\dictionary.txt");
        System.out.println(sc.checkSpelling("hello lyn, you are the cutest wiblet!"));
        
    }
}


